# Supports Organizer.rb
=begin rdoc
= Voodoo Camera Tracker Importer
Copyright 2009-2011, Chris Fullmer
All Rights Reserved

== Disclaimer
THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

== License
This software is distributed under the Smustard End User License Agreement
http://www.smustard.com/eula

== Information
Author:: Chris Fullmer
Organization:: www.ChrisFullmer.com and distributed on www.sketchucation.com
Name:: Voodoo Camera Tracker Importer
Version:: v1.1
SU Version:: Developed on SU 7.1, and 8.0, should be compatible with 6 and 5 at least.
Date:: March 5th, 2011
Description:: This plugin makes a point cloud in Sketchup from a voodoo camera tracking software text file.  It also places the camera and makes a scene for each frame of the video.  Please check out Voodoo camera tracker for info on how to use their software. 
Usage:: (File Menu > Import > voodoo) file type or though (Plugins > Chris Fullmer Tools > Voodoo Importer).  Please go to http://forums.sketchucation.com for a brief tutorial on how to use this.
History::
1.0:: 2009-11-29
   * First release
   * Imports the text file exported from Voodoo Camera Tracking software
   * Makes scenes for each camera frame
   * Adds construction points for the feature points from Voodoo.
1.1:: 2011-03-05
   * Fixed a small bug where the this plugin was always attempting to open a file from a hardwired location when the plugin was run from the plugins menu (as opposed to being run from the "Import" window.
=end

require 'sketchup.rb'

class Clf_voodoo_importer < Sketchup::Importer

	def description
		return "Voodoo Camera Tracker Importer (*.txt)"
	end

	def file_extension
		return "txt"
	end

	def id
		return "com.sketchup.importers.voodoo_tracker"
	end

	def supports_options?
		return false
	end

	# I've decided not to include this in my script for now
	def do_options
		my_settings = UI.inputbox(['Frames Per Second'], ['29.97'], "Import Options")
		@fps = my_settings[0]
	end


	def load_file(file_path, status)
		startup(file_path)
		return 0 # 0 is the code for a successful import
	end


	def startup( pts_file )
		model = Sketchup.active_model
		ents = model.active_entities
		@pages = Sketchup.active_model.pages
		@view = Sketchup.active_model.active_view
		cam_grp = ents.add_group
		cam_ents = cam_grp.entities
		model_grp = ents.add_group
		model_ents = model_grp.entities
		model.start_operation "Camera Matcher"
		pos = []
		line_info = []
		@pg_num = 1
		iw = []
		ih = []
		@pg_name = "f"
		prompts = ["How many Frames Per Second?"]
		defaults = [29.97]
		results = UI.inputbox (prompts, defaults, "Frames Per Second")
		@fps = results[0]

		IO.foreach(pts_file) do |line|
			 unless (line[0].chr) == '#'
				line_info = ( line.split(/\s+/) ).collect {|num| num.to_f }
				case line_info.length
				when 28
					pos = [-line_info[0], -line_info[2], -line_info[1]]
					fovw = line_info[21]
					iw = line_info[16]
					ih = line_info[17]
					fov = (ih.to_f / iw.to_f) * fovw
					cam_ents.add_cpoint [pos[0], pos[1], pos[2]]
					up = Geom::Vector3d.new(line_info[25].to_f, line_info[27].to_f, line_info[26].to_f)
					direction = Geom::Vector3d.new(-line_info[3].to_f, -line_info[5].to_f, -line_info[4].to_f)
					make_scenes( line_info, pos, direction, up, fovw )
				when 3
					pos = [-line_info[0], -line_info[2], -line_info[1]]
					model_ents.add_cpoint [pos[0], pos[1], pos[2]]
				when 0
					puts "No Info"
				else
					puts "Unknown info - #{line_info.length} units long"
				end
			 end
		end
		model.commit_operation
	end

	def make_scenes( line_info, pos, direction, up, fov )
		cam = Sketchup::Camera.new( [pos[0],pos[1],pos[2]], direction, up )
		cam.fov = fov
		cam.aspect_ratio = line_info[16] / line_info[17]
		@view.camera=cam
		page = @pages.add @pg_name+@pg_num.to_s, 1
		@pg_num = @pg_num + 1
		page.transition_time=(1.0/@fps.to_f)
		page.delay_time=0
	end

	def get_file
		pts_file = UI.openpanel( "Open Voodoo Text File", "", "" )
		if pts_file
			startup(pts_file)
		end
	end
	
end


Sketchup.register_importer(Clf_voodoo_importer.new)




#------Menu System---------------------------

clf_voodoo = Clf_voodoo_importer.new

if( $submenu )
	organizer = $submenu 
	if !file_loaded?(__FILE__)
		organizer.add_item("Voodoo Camera Tracker Importer") { clf_voodoo.get_file }
	end
else
	if !file_loaded?('clf_menu_loader')
		$clf_tools_menu = UI.menu("Plugins").add_submenu("Chris Fullmer Tools")
	end
#	------New menu Items---------------------------
	if !file_loaded?('clf_youtube_loader')
		$clf_youtube_menu = $clf_tools_menu.add_submenu("-=YouTube Video Tutorials For My Scripts=-")
		$clf_youtube_menu.add_item("My YouTube Channel"){UI.openURL "http://www.youtube.com/user/ChrisFullmer"}
	end
	if !file_loaded?('clf_myscipts_loader')
		$clf_tools_menu.add_item("-=All My Plugins I've Written=-"){UI.openURL "http://forums.sketchucation.com/search.php?keywords=plugin&terms=all&author=Chris+Fullmer&sc=1&sf=titleonly&sr=topics&sk=t&sd=d&st=0&ch=300&t=0&submit=Search"}
	end
#	------YouTube for this Script------------------
	$clf_youtube_menu.add_item("Voodoo Importer Information"){UI.openURL "http://www.chrisfullmer.com/plugins/clf_voodoo_importer.html"}		
#	------Donate to the Author--------------------
	if !file_loaded?('clf_paypal_loader')
		$clf_tools_menu.add_item("-=Donate $$$ to Chris via PayPal=-"){UI.openURL "http://www.chrisfullmer.com/plugins/donate.html"}
		$clf_tools_menu.add_separator
	end
#	------------------------------------------------
	
	if !file_loaded?(__FILE__)
		$clf_tools_menu.add_item("Voodoo Camera Tracker Importer") { clf_voodoo.get_file }
	end
end

file_loaded('clf_myscipts_loader')
file_loaded('clf_paypal_loader')
file_loaded('clf_youtube_loader')
file_loaded('clf_menu_loader')
file_loaded(__FILE__)